<?php
/**
 * Module Name: User Country Flags
 * Description: Display visitor country flags in comments & enable the <code>{FLAG}</code> variable in comment-related widgets. Relies on the <a href="http://firestats.cc/wiki/ip2c">IP2C database</a>
 * Version: 1.0
 * Author: digitalnature
 * Author URI: http://digitalnature.eu
 * Auto Enable: no 
 */



// class name must follow this pattern (AtomMod + directory name)
class AtomModCountryFlags extends AtomMod{

  // available public variables from parent class:
  //
  // $this->url  - this module's url path
  // $this->dir  - this module's directory

  // required method
  public function init(){

    // hooks
    atom()->add('comment_author',                     array(&$this, 'commentAuthor'), 10, 3);
    atom()->add('widget_comments_keywords',           array(&$this, 'widgetTemplateVars'), 10, 3);
    atom()->add('widget_top_commenters_keywords',     array(&$this, 'widgetTemplateVars'), 10, 3);

    add_action('wp_enqueue_scripts',                  array(&$this, 'assets'));
    add_filter('comment_class',                       array(&$this, 'commentClass'));

  }



  // gets the country info based on a IP address
  private function getCountry($ip) {
    $ip2c = wp_cache_get("atom_ip2c");
    if($ip2c === false){
      $ip2c = new ip2country($this->dir.'/ip-to-country.bin');
      wp_cache_set("atom_ip2c", $ip2c);
    }

    $country = $ip2c->get_country($ip);
    if($country){
      $code = strtolower($country['id2']);
      $name = ucwords(strtolower($country['name']));
      return array('code' => $code, 'name' => $name);
    }else{
      return false;
    }
  }



  // country flags css
  public function assets(){
    wp_enqueue_style(ATOM.'-flags', $this->url.'/flags.css', array(ATOM.'-core'), atom()->get('theme_version'));
  }



  // add country info to comment class
  public function commentClass($classes){
    global $comment;

    if(!isset($comment->country))
      $comment->country = $this->getCountry($comment->comment_author_IP);

    if(!empty($comment->country)) $classes[] = "country-{$comment->country['code']}";
    return $classes;
  }



  // country flag next to comment author link
  public function commentAuthor($output, $comment){
    if(!isset($comment->country))
      $comment->country = $this->getCountry($comment->comment_author_IP);

    if(!empty($comment->country))
      $output .= "<abbr title=\"{$comment->country['name']}\" class=\"flag {$comment->country['code']}\">&nbsp;</abbr>";

    return $output;
  }

  // add {FLAG} template variable to recent comments and top commenters widgets
  public function widgetTemplateVars($vars, $comm, $args){
    $vars['FLAG'] = ($country = $this->getCountry($comm->comment_author_IP)) ? '<abbr title="'.$country['name'].'" class="flag '.$country['code'].'">&nbsp;</abbr>' : '';
    return $vars;
  }

}



// ip2c class
if(!class_exists('ip2country')){

  define('IP2C_MAX_INT', 0x7fffffff);

  class ip2country{
    var $m_active = false;
    var $m_file;
    var $m_firstTableOffset;
    var $m_numRangesFirstTable;
    var $m_secondTableOffset;
    var $m_numRangesSecondTable;
    var $m_countriesOffset;
    var $m_numCountries;

    function ip2country($bin_file = 'ip-to-country.bin'){
      $this->m_file = fopen($bin_file, "rb");

      if(!$this->m_file){
        trigger_error('Error loading '.$bin_file);
        if(defined('UNIT_TEST')) exit(1);
        return;
      }

      $f = $this->m_file;
      $sig = fread($f, 4);

      if($sig != 'ip2c'){
        trigger_error("file $bin_file has incorrect signature");
        if(defined('UNIT_TEST')) exit(1);
        return;
      }
      $v = $this->readInt();
      if($v != 2){
        trigger_error("file $bin_file has incorrect format version ($v)");
        if(defined('UNIT_TEST')) exit(1);
        return;
      }

      $this->m_firstTableOffset = $this->readInt();
      $this->m_numRangesFirstTable = $this->readInt();
      $this->m_secondTableOffset = $this->readInt();
      $this->m_numRangesSecondTable = $this->readInt();
      $this->m_countriesOffset = $this->readInt();
      $this->m_numCountries = $this->readInt();
      $this->m_active = true;
    }

    function get_country($ip){
      if(!$this->m_active) return false;

      $int_ip =  ip2long($ip);

      // happens on 64bit systems
      if($int_ip > IP2C_MAX_INT){
        // shift to signed int32 value
        $int_ip -= IP2C_MAX_INT;
        $int_ip -= IP2C_MAX_INT;
        $int_ip -= 2;
      }

      if($int_ip >= 0){
        $key = $this->find_country_code($int_ip, 0, $this->m_numRangesFirstTable, true);
      }
      else{
        $nip = (int)($int_ip + IP2C_MAX_INT + 2); // the + 2 is a bit wierd, but required.
        $key = $this->find_country_code($nip, 0, $this->m_numRangesSecondTable, false);
      }

      if($key == false || $key == 0) return false; else return $this->find_country_key($key,0, $this->m_numCountries);
    }

    function find_country_code($ip, $startIndex, $endIndex, $firstTable, $d = 0){
      while(1){
        $middle = (int)(($startIndex + $endIndex) / 2);
        $mp = $this->getPair($middle, $firstTable);
        $mip = $mp['ip'];

        if($ip < $mip){
          if($startIndex + 1 == $endIndex) return false; // not found
          $endIndex = $middle;
          continue;
        }else
          if($ip > $mip){
            $np = $this->getPair($middle+1, $firstTable);
            if($ip < $np['ip']){
              return $mp['key'];
            }else{
              if ($startIndex + 1 == $endIndex) return false; // not found
              $startIndex = $middle;
              continue;
            }
          }
          else{ // ip == mip
            return $mp['key'];
          }
      }
    }

    function find_country($code){
      if(!$this->m_active) return false;
      $c = strtoupper($code);
      $c1 = $c[0];
      $c2 = $c[1];
      $key = ord($c1) * 256 + ord($c2);
      return $this->find_country_key($key, 0, $this->m_numCountries);
    }

    function find_country_key($code, $startIndex, $endIndex){
      $d = 0;
      while(1){
        if($d > 20){
          trigger_error("IP2Country : Internal error - endless loop detected, code = $code");
          return false;
        }

        $d++;
        $middle = (int)(($startIndex + $endIndex) / 2);
        $mc = $this->get_country_code($middle);

        if($mc == $code){ // found.
          return $this->load_country($middle);
        }
        else
          if($code > $mc){
            if($middle + 1 == $endIndex){
              $nc = $this->get_country_code($middle);
              if($nc == $code) return $this->load_country($middle); else return false;
            }
            $startIndex = $middle;
            continue;
            //return $this->find_country_key($code, $middle, $endIndex, ++$d);
          } else {  // $code < $mc
            if($startIndex + 1 == $middle){
              $nc = $this->get_country_code($startIndex);
              if($nc == $code) return $this->load_country($startIndex); else return false;
            }
            $endIndex = $middle;
            continue;
          }
      }
    }

    function load_country($index){
      $offset = $this->m_countriesOffset + $index * 10;
      fseek($this->m_file, $offset);

      $id2c = $this->readCountryKey();
      $id3c = $this->read3cCode();
      $nameOffset = $this->readInt();

      fseek($this->m_file, $nameOffset);

      $len = $this->readShort();
      $name = '';
      if($len != 0) $name = fread($this->m_file, $len);

      return array("id2" => $id2c, "id3" => $id3c, "name" => $name);
    }

    function get_country_code($index){
      $offset = $this->m_countriesOffset + $index * 10;

      fseek($this->m_file, $offset);
      $a = unpack('n', fread($this->m_file, 2));

      return $a[1];
    }

    function getPair($index, $firstTable){
      $offset = 0;
      if($firstTable){
        if($index > $this->m_numRangesFirstTable) return array('key'=>false,'ip'=>0);
        $offset = $this->m_firstTableOffset + $index * 6;
      } else {
        if($index > $this->m_numRangesSecondTable) return array('key'=>false,'ip'=>0);
        $offset = $this->m_secondTableOffset + $index * 6;
      }

      fseek($this->m_file, $offset);
      $p = unpack('Nip/nkey', fread($this->m_file, 6));

      return $p;
    }

    function readShort(){
      $a = unpack('n', fread($this->m_file, 2));
      return $a[1];
    }

    function read3cCode(){
      fread($this->m_file, 1);
      $d = fread($this->m_file, 3);
      return $d != '   ' ? $d : '';
    }

    function readCountryKey(){
      return fread($this->m_file, 2);
    }

    function readInt(){
      $a = unpack('N', fread($this->m_file, 4));
      return $a[1];
    }
  }

}




?>